from enum import Enum


class HttpMethod(Enum):
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"
    COPY = "COPY"
    HEAD = "HEAD"
    OPTIONS = "OPTIONS"
    LINK = "LINK"
    UNLINK = "UNLINK"
    PURGE = "PURGE"
    LOCK = "LOCK"
    UNLOCK = "UNLOCK"
    PROPFIND = "PROPFIND"
    VIEW = "VIEW"